@@ -556,6 +556,7 @@ pub trait CSharpWriter {
         indented!(w, [_], r#"{{"#)?;
         indented!(w, [_ _], r#"get"#)?;
         indented!(w, [_ _], r#"{{"#)?;
+        indented!(w, [_ _ _], r#"if (i >= Count) throw new IndexOutOfRangeException();"#)?;
         indented!(w, [_ _ _], r#"var size = Marshal.SizeOf(typeof({}));"#, type_string)?;
         indented!(w, [_ _ _], r#"var ptr = new IntPtr(data.ToInt64() + i * size);"#)?;
         indented!(w, [_ _ _], r#"return Marshal.PtrToStructure<{}>(ptr);"#, type_string)?;
@@ -576,18 +577,12 @@ pub trait CSharpWriter {
         indented!(w, [_], r#"}}"#)?;
 
         // Count
-        indented!(w, [_], r#"public int Count"#)?;
-        indented!(w, [_], r#"{{"#)?;
-        indented!(w, [_ _], r#"get"#)?;
-        indented!(w, [_ _], r#"{{"#)?;
-        indented!(w, [_ _ _], r#"return (int) len;"#)?;
-        indented!(w, [_ _], r#"}}"#)?;
-        indented!(w, [_], r#"}}"#)?;
+        indented!(w, [_], r#"public int Count => (int) len;"#)?;
 
         // GetEnumerator
         indented!(w, [_], r#"public IEnumerator<{}> GetEnumerator()"#, type_string)?;
         indented!(w, [_], r#"{{"#)?;
-        indented!(w, [_ _], r#"for (int i = 0; i < (int)len; ++i)"#)?;
+        indented!(w, [_ _], r#"for (var i = 0; i < (int)len; ++i)"#)?;
         indented!(w, [_ _], r#"{{"#)?;
         indented!(w, [_ _ _], r#"yield return this[i];"#)?;
         indented!(w, [_ _], r#"}}"#)?;
@@ -634,12 +629,14 @@ pub trait CSharpWriter {
         indented!(w, [_], r#"{{"#)?;
         indented!(w, [_ _], r#"get"#)?;
         indented!(w, [_ _], r#"{{"#)?;
+        indented!(w, [_ _ _], r#"if (i >= Count) throw new IndexOutOfRangeException();"#)?;
         indented!(w, [_ _ _], r#"var size = Marshal.SizeOf(typeof({}));"#, type_string)?;
         indented!(w, [_ _ _], r#"var ptr = new IntPtr(data.ToInt64() + i * size);"#)?;
         indented!(w, [_ _ _], r#"return Marshal.PtrToStructure<{}>(ptr);"#, type_string)?;
         indented!(w, [_ _], r#"}}"#)?;
         indented!(w, [_ _], r#"set"#)?;
         indented!(w, [_ _], r#"{{"#)?;
+        indented!(w, [_ _ _], r#"if (i >= Count) throw new IndexOutOfRangeException();"#)?;
         indented!(w, [_ _ _], r#"var size = Marshal.SizeOf(typeof({}));"#, type_string)?;
         indented!(w, [_ _ _], r#"var ptr = new IntPtr(data.ToInt64() + i * size);"#)?;
         indented!(w, [_ _ _], r#"Marshal.StructureToPtr<{}>(value, ptr, false);"#, type_string)?;
@@ -660,18 +657,12 @@ pub trait CSharpWriter {
         indented!(w, [_], r#"}}"#)?;
 
         // Count
-        indented!(w, [_], r#"public int Count"#)?;
-        indented!(w, [_], r#"{{"#)?;
-        indented!(w, [_ _], r#"get"#)?;
-        indented!(w, [_ _], r#"{{"#)?;
-        indented!(w, [_ _ _], r#"return (int) len;"#)?;
-        indented!(w, [_ _], r#"}}"#)?;
-        indented!(w, [_], r#"}}"#)?;
+        indented!(w, [_], r#"public int Count => (int) len;"#)?;
 
         // GetEnumerator
         indented!(w, [_], r#"public IEnumerator<{}> GetEnumerator()"#, type_string)?;
         indented!(w, [_], r#"{{"#)?;
-        indented!(w, [_ _], r#"for (int i = 0; i < (int)len; ++i)"#)?;
+        indented!(w, [_ _], r#"for (var i = 0; i < (int)len; ++i)"#)?;
         indented!(w, [_ _], r#"{{"#)?;
         indented!(w, [_ _ _], r#"yield return this[i];"#)?;
         indented!(w, [_ _], r#"}}"#)?;
@@ -793,7 +784,7 @@ pub trait CSharpWriter {
                 indented!(w, [_], r#"var rval = {};"#, fn_call)?;
                 indented!(w, [_], r#"if (rval != {}.{})"#, e.the_enum().rust_name(), e.success_variant().name())?;
                 indented!(w, [_], r#"{{"#)?;
-                indented!(w, [_ _], r#"throw new Exception("Something went wrong");"#)?;
+                indented!(w, [_ _], r#"throw new Exception($"Something went wrong: {{rval}}");"#)?;
                 indented!(w, [_], r#"}}"#)?;
             }
             CType::Primitive(PrimitiveType::Void) => {
